#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pymirror', 'pymirror.experimental']

package_data = \
{'': ['*'], 'pymirror': ['.addons/*', 'data/*']}

entry_points = \
{'console_scripts': ['pymirror = pymirror.run:main']}

setup(name='pymirror',
      version='0.3.3',
      description='pymirror',
      author=None,
      author_email='Alyetama <56323389+Alyetama@users.noreply.github.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      entry_points=entry_points,
     )
